/*
 *	CERCA.C
 *	LaMoSca (LAboratorio di MOtori per SCAcchi)
 *	versione 0.06
 *	2001 Pietro Valocchi
 */


#include <stdio.h>
#include "Common.h"
#include "Extern.h"


/* MaxSearch()  massimizza il valore della posizione del programma */
BOOL MaxSearch(Mossa *M, int *ValoreMassimo) {

	int  i, ValoreCorrente;
	BOOL Trovata = FALSE;

	/* genera mosse per il programma */
	GeneraPseudo();

	/* verifica che il re non sia in scacco */
	for (i = InizioMosse[Livello-1]; i < InizioMosse[Livello]; i++) {
		if (B[StackMosse[i].A].Pezzo == RE) {
			/* la mossa del livello precedente ha lasciato il re in scacco */
			return FALSE;
		}
	}

	/* valuta la posizione se siamo al livello massimo */
	if (Livello == (LivelloMassimo + 1)) {
		*ValoreMassimo = Valuta();
		return TRUE;
	} else {

		/* prova tutte le mosse fino a trovare la migliore */
		if (LogFile) fprintf(LogFile, "\nMax livello %d: ", Livello);

		*ValoreMassimo = - INFINITO;

		for (i = InizioMosse[Livello-1]; i < InizioMosse[Livello]; i++) {

			/* esegui la mossa */
			Muovi(&StackMosse[i]);

			/* trova la risposta migliore per l'avversario */
			if (MinSearch(M, &ValoreCorrente)) {
				Trovata = TRUE;

				/* massimizza il valore della posizione del programma */
				if (ValoreCorrente > *ValoreMassimo) {
					*ValoreMassimo = ValoreCorrente;
					/* salva la mossa scelta */
					if (Livello == 2) {
						M->A  = StackMosse[i].A;
						M->Da = StackMosse[i].Da;
					}
				}
			}
			/* ripristina la mossa */
			TornaIndietro(&StackMosse[i]);

			if (LogFile)
				fprintf(LogFile, "M%d:%c%d%c%d=%d ", Livello, 
				COLONNA(StackMosse[i].Da)+ 'a', 8 - RIGA(StackMosse[i].Da), 
				COLONNA(StackMosse[i].A) + 'a', 8 - RIGA(StackMosse[i].A),
				ValoreCorrente);
		}

		if (LogFile) {
			fprintf(LogFile, "\nMax L%d: T=%d", Livello, Trovata);
			if (Trovata == TRUE) {
				if (Livello == 1)
					fprintf(LogFile, ", M=%c%d%c%d", 
					COLONNA(M->Da)+ 'a', 8 - RIGA(M->Da), 
					COLONNA(M->A) + 'a', 8 - RIGA(M->A));
				fprintf(LogFile, ", V=%d", *ValoreMassimo);
			}
			fprintf(LogFile, "\n");
		}
	}

	return Trovata;
}


/* MinSearch() sceglie la mossa che minimizza la posizione dell'avversario */
BOOL MinSearch(Mossa *M, int *ValoreMinimo) {

	int  i, ValoreCorrente;
	BOOL Trovata = FALSE;

	/* genera mosse per il programma */
	GeneraPseudo();

	/* verifica che il re non sia in scacco */
	for (i = InizioMosse[Livello-1]; i < InizioMosse[Livello]; i++) {
		if (B[StackMosse[i].A].Pezzo == RE) {
			return FALSE;
		}
	}

	/* valuta la posizione se siamo al livello massimo */
	if (Livello == (LivelloMassimo + 1)) {
		*ValoreMinimo = Valuta();
		return TRUE;
	} else {

		/* prova tutte le mosse dell'avversario fino a trovare quella a valore minimo */
		if (LogFile) fprintf(LogFile, "\nmin livello %d: ", Livello);

		*ValoreMinimo = INFINITO;

		for (i = InizioMosse[Livello-1]; i < InizioMosse[Livello]; i++) {

			/* esegui la mossa */
			Muovi(&StackMosse[i]);

			/* trova la mossa migliore del programma */
			if (MaxSearch(M, &ValoreCorrente)) {
				Trovata = TRUE;

				/* minimizza il punteggio delle risposte dell'avversario */
				if (ValoreCorrente < *ValoreMinimo) {
					*ValoreMinimo = ValoreCorrente;
					/* salva la mossa scelta */
					if (Livello == 2) {
						M->A  = StackMosse[i].A;
						M->Da = StackMosse[i].Da;
					}
				}
			}
			/* ripristina la mossa */
			TornaIndietro(&StackMosse[i]);

			if (LogFile)
				fprintf(LogFile, "m%d:%c%d%c%d=%d ", Livello, 
				COLONNA(StackMosse[i].Da)+ 'a', 8 - RIGA(StackMosse[i].Da), 
				COLONNA(StackMosse[i].A) + 'a', 8 - RIGA(StackMosse[i].A),
				ValoreCorrente);
		}

		if (LogFile) {
			fprintf(LogFile, "\nmin L%d: T=%d", Livello, Trovata);
			if (Trovata == TRUE) {
				if (Livello == 1)
					fprintf(LogFile, ", M=%c%d%c%d", 
					COLONNA(M->Da)+ 'a', 8 - RIGA(M->Da), 
					COLONNA(M->A) + 'a', 8 - RIGA(M->A));
				fprintf(LogFile, ", V=%d", *ValoreMinimo);
			}
			fprintf(LogFile, "\n");
		}
	}

	return Trovata;
}

